from gen3.auth import Gen3Auth


class Auth(Gen3Auth):
    """
    Class for Gen3 Authentication inherited from the offical Gen3 API's Gen3 Auth Helper
    https://gen3sdk-python.readthedocs.io/en/latest/auth.html
    """

    __doc__ += Gen3Auth.__doc__
    pass
